package src.Generics;

import src.DrawningObjects.DrawningAirplane;
import src.MovementStrategy.IMoveableObject;

import java.awt.*;
import java.awt.image.BufferedImage;

public class AirplaneGenericCollection<T extends DrawningAirplane, U extends IMoveableObject> {
    private final int _pictureWidth;

    private final int _pictureHeight;

    private final int _placeSizeWidth = 166;

    private final int _placeSizeHeight = 160;

    private final SetGeneric<T> _collection;

    public AirplaneGenericCollection(int picWidth, int picHeight){
        int width = picWidth / _placeSizeWidth;
        int height = picHeight / _placeSizeHeight;
        _pictureWidth = picWidth-202;
        _pictureHeight = picHeight;
        _collection = new SetGeneric<T>(width * height);
    }

    public int Insert(T obj){
        if (obj == null)
        {
            return -1;
        }
        return _collection.Insert(obj);
    }

    public boolean Remove(int position){
        return _collection.Remove(position);
    }
    public U GetU(int pos){
        T ans = _collection.Get(pos);
        if(ans == null)
            return null;
        return (U)ans.GetMoveableObject();
    }

    private void DrawBackground(Graphics g)
    {
        g.setColor(Color.BLACK);
        for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
        {
            for (int j = 0; j < _pictureHeight / _placeSizeHeight +
                    1; ++j)
            {
                g.drawLine(i * _placeSizeWidth, j *
                        _placeSizeHeight, i * _placeSizeWidth + _placeSizeWidth / 2, j *
                        _placeSizeHeight);
            }
            g.drawLine(i * _placeSizeWidth, 0, i *
                    _placeSizeWidth, _pictureHeight / _placeSizeHeight * _placeSizeHeight);
        }
    }

    private void DrawObjects(Graphics g)
    {
        for (int i = 0; i < _collection.Count; i++)
        {
            DrawningAirplane airplane = _collection.Get(i);
            if (airplane != null)
            {
                int inRow = _pictureWidth / _placeSizeWidth;
                airplane.SetPosition(_pictureWidth - _placeSizeWidth - (i % inRow * _placeSizeWidth) - _placeSizeWidth /10 * 2 , (_collection.Count / inRow - 1 - i / inRow) * _placeSizeHeight + (_placeSizeHeight - _placeSizeHeight * 170 / 1000) / 2);
                airplane.DrawAirplane((Graphics2D) g);
            }
        }
    }
    public BufferedImage ShowAirplanes()
    {
        BufferedImage bmp = new BufferedImage(_pictureWidth, _pictureHeight, BufferedImage.TYPE_4BYTE_ABGR);
        Graphics gr = bmp.createGraphics();
        DrawBackground(gr);
        DrawObjects(gr);
        return bmp;
    }
}
